/**
 * This class was created by BrassGoggledCoders modding team.
 * This class is available as part of the Steamcraft 2 Mod for Minecraft.
 *
 * Steamcraft 2 is open-source and is distributed under the MMPL v1.0 License.
 * (http://www.mod-buildcraft.com/MMPL-1.0.txt)
 *
 * Steamcraft 2 is based on the original Steamcraft Mod created by Proloe.
 * Steamcraft (c) Proloe 2011
 * (http://www.minecraftforum.net/topic/251532-181-steamcraft-source-code-releasedmlv054wip/)
 *
 */
package xyz.brassgoggledcoders.boilerplate.common.entity;

import net.minecraft.block.Block;
import net.minecraft.entity.Entity;
import net.minecraft.entity.EntityLivingBase;
import net.minecraft.entity.IProjectile;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.util.*;
import net.minecraft.world.World;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;

import java.util.List;

/**
 * @author Surseance (Johnny Eatmon)
 *
 */
public abstract class BaseShootableEntity extends Entity implements IProjectile
{
	protected int xTile = -1;
	protected int yTile = -1;
	protected int zTile = -1;

	protected Block inTile;

	protected Entity shootingEntity;

	protected int timeTillDeath;
	protected int flyTime;

	protected int accuracy;

	public BaseShootableEntity(World world)
	{
		super(world);
		this.func_70105_a(1F, 1F);
		this.field_70155_l = 10.0D;
	}

	/**
	 * Checks if the entity is in range to render by using the past in distance
	 * and comparing it to its average edge length * 64 * renderDistanceWeight
	 * Args: distance
	 */
	@Override
	@SideOnly(Side.CLIENT)
	public boolean func_70112_a(double distance)
	{
		double d1 = this.func_174813_aQ().func_72320_b() * 4.0D;
		d1 *= 64.0D;
		return distance < (d1 * d1);
	}

	public BaseShootableEntity(World world, double dx, double dy, double dz)
	{
		super(world);
		this.func_70105_a(1F, 1F);
		this.field_70155_l = 10.0D;
		this.func_70105_a(0.5F, 0.5F);
		this.func_70107_b(dx, dy, dz);
	}

	public BaseShootableEntity(World world, EntityLivingBase shooter, EntityLivingBase target, float frotY, float frotP)
	{
		super(world);
		this.field_70155_l = 10.0D;
		this.shootingEntity = shooter;

		this.field_70163_u = (shooter.field_70163_u + shooter.func_70047_e()) - 0.10000000149011612D;
		double dx = target.field_70165_t - shooter.field_70165_t;
		double dy = (target.func_174813_aQ().field_72338_b + (target.field_70131_O / 3.0F)) - this.field_70163_u;
		double dz = target.field_70161_v - shooter.field_70161_v;
		double magnitude = MathHelper.func_76133_a((dx * dx) + (dz * dz));

		if (magnitude >= 1.0E-7D)
		{
			float fx = (float) ((Math.atan2(dz, dx) * 180.0D) / Math.PI) - 90.0F;
			float fy = (float) -((Math.atan2(dy, magnitude) * 180.0D) / Math.PI);
			double dlx = dx / magnitude;
			double dlz = dz / magnitude;
			this.func_70012_b(shooter.field_70165_t + dlx, this.field_70163_u, shooter.field_70161_v + dlz, fx, fy);
			float height = (float) magnitude * 0.2F;
			this.func_70186_c(dx, dy + height, dz, frotY, frotP);
		}
	}

	public BaseShootableEntity(World world, EntityLivingBase shooter, int damage, int accuracy)
	{
		super(world);
		this.field_70155_l = 10.0D;
		this.shootingEntity = shooter;

		this.func_70105_a(0.5F, 0.5F);
		this.func_70012_b(shooter.field_70165_t, shooter.field_70163_u + shooter.func_70047_e(), shooter.field_70161_v, shooter.field_70177_z, shooter.field_70125_A);
		this.field_70165_t -= MathHelper.func_76134_b((this.field_70177_z / 180.0F) * (float) Math.PI) * 0.16F;
		this.field_70163_u -= 0.10000000149011612D;
		this.field_70161_v -= MathHelper.func_76126_a((this.field_70177_z / 180.0F) * (float) Math.PI) * 0.16F;
		this.func_70107_b(this.field_70165_t, this.field_70163_u, this.field_70161_v);
		this.field_70159_w = -MathHelper.func_76126_a((this.field_70177_z / 180.0F) * (float) Math.PI)
				* MathHelper.func_76134_b((this.field_70125_A / 180.0F) * (float) Math.PI);
		this.field_70179_y = MathHelper.func_76134_b((this.field_70177_z / 180.0F) * (float) Math.PI)
				* MathHelper.func_76134_b((this.field_70125_A / 180.0F) * (float) Math.PI);
		this.field_70181_x = -MathHelper.func_76126_a((this.field_70125_A / 180.0F) * (float) Math.PI);
		this.func_70186_c(this.field_70159_w, this.field_70181_x, this.field_70179_y, 1F, accuracy);
		this.accuracy = accuracy;
	}

	@Override
	public void func_70186_c(double dx, double dy, double dz, float frotY, float frotP)
	{
		int accuracy = this.accuracy;
		float f2 = MathHelper.func_76133_a((dx * dx) + (dy * dy) + (dz * dz));
		dx /= f2;
		dy /= f2;
		dz /= f2;
		dx += (this.field_70146_Z.nextGaussian() * 0.0034999998323619365D * frotP * accuracy) / 5;
		dy += (this.field_70146_Z.nextGaussian() * 0.0034999998323619365D * frotP * accuracy) / 5;
		dz += (this.field_70146_Z.nextGaussian() * 0.0034999998323619365D * frotP * accuracy) / 5;
		dx *= frotY;
		dy *= frotY;
		dz *= frotY;
		this.field_70159_w = dx;
		this.field_70181_x = dy;
		this.field_70179_y = dz;
		float magnitude = MathHelper.func_76133_a((dx * dx) + (dz * dz));
		this.field_70126_B = this.field_70177_z = (float) ((Math.atan2(dx, dz) * 180D) / 3.1415927410125732D);
		this.field_70127_C = this.field_70125_A = (float) ((Math.atan2(dy, magnitude) * 180D) / 3.1415927410125732D);
		this.timeTillDeath = 0;
	}

	/* TODO Set Position
	@Override
	@SideOnly(Side.CLIENT)
	public void setPositionAndRotation2(double dx, double dy, double dz, float frotY, float frotP, int i)
	{
		this.setPosition(dx, dy, dz);
		this.setRotation(frotY, frotP);
	}*/

	@Override
	@SideOnly(Side.CLIENT)
	public void func_70016_h(double dx, double dy, double dz)
	{
		this.field_70159_w = dx;
		this.field_70181_x = dy;
		this.field_70179_y = dz;

		if ((this.field_70127_C == 0.0F) && (this.field_70126_B == 0.0F))
		{
			float magnitude = MathHelper.func_76133_a((dx * dx) + (dz * dz));
			this.field_70126_B = this.field_70177_z = (float) ((Math.atan2(dx, dz) * 180D) / 3.1415927410125732D);
			this.field_70127_C = this.field_70125_A = (float) ((Math.atan2(dy, magnitude) * 180D) / 3.1415927410125732D);
		}
	}

	@Override
	public void func_70071_h_()
	{
		super.func_70071_h_();

		if (this.flyTime > 1000)
		{
			this.func_70106_y();
		}

		if ((this.field_70127_C == 0.0F) && (this.field_70126_B == 0.0F))
		{
			float magnitude = MathHelper.func_76133_a((this.field_70159_w * this.field_70159_w) + (this.field_70179_y * this.field_70179_y));
			this.field_70126_B = this.field_70177_z = (float) ((Math.atan2(this.field_70159_w, this.field_70179_y) * 180D) / 3.1415927410125732D);
			this.field_70127_C = this.field_70125_A = (float) ((Math.atan2(this.field_70181_x, magnitude) * 180D) / 3.1415927410125732D);
		}
		this.flyTime++;

		Vec3 posVector = new Vec3(this.field_70165_t, this.field_70163_u, this.field_70161_v);
		Vec3 velVector = new Vec3(this.field_70165_t + this.field_70159_w, this.field_70163_u + this.field_70181_x, this.field_70161_v + this.field_70179_y);
		MovingObjectPosition mop = this.field_70170_p.func_72901_a(posVector, velVector, false);

		posVector = new Vec3(this.field_70165_t, this.field_70163_u, this.field_70161_v);
		velVector = new Vec3(this.field_70165_t + this.field_70159_w, this.field_70163_u + this.field_70181_x, this.field_70161_v + this.field_70179_y);

		if (mop != null)
		{
			velVector = new Vec3(mop.field_72307_f.field_72450_a, mop.field_72307_f.field_72448_b, mop.field_72307_f.field_72449_c);
		}

		Entity entity = null;
		List<?> entList = this.field_70170_p.func_72839_b(this,
				this.func_174813_aQ().func_72321_a(this.field_70159_w, this.field_70181_x, this.field_70179_y).func_72314_b(1.0D, 1.0D, 1.0D));
		double distance = 0.0D;

		for (Object obj : entList)
		{
			Entity collidableEnt = (Entity) obj;

			if (!collidableEnt.func_70067_L() || ((collidableEnt == this.shootingEntity) && (this.flyTime < 5)))
			{
				continue;
			}

			float amount = 0.3F;
			AxisAlignedBB aaBB = collidableEnt.func_174813_aQ().func_72314_b(amount, amount, amount);
			MovingObjectPosition objectInVector = aaBB.func_72327_a(posVector, velVector);

			if (objectInVector == null)
			{
				continue;
			}

			double distanceToObject = posVector.func_72438_d(objectInVector.field_72307_f);

			if ((distanceToObject < distance) || (distance == 0.0D))
			{
				entity = collidableEnt;
				distance = distanceToObject;
			}
		}

		if (entity != null)
		{
			mop = new MovingObjectPosition(entity);
		}

		if (mop != null)
		{
			if (mop.field_72308_g != null)
			{
				if (mop.field_72308_g != this.shootingEntity)
				{
					this.onHitEntity(mop.field_72308_g);
					this.field_70159_w *= 0.10000000149011612D;
					this.field_70181_x *= 0.10000000149011612D;
					this.field_70179_y *= 0.10000000149011612D;
					this.flyTime = 0;
					this.func_70106_y();
				}
			}
			else
			{
				this.inTile = this.field_70170_p.func_180495_p(mop.func_178782_a()).func_177230_c();

				this.onHitBlock(this.inTile, mop);
			}
		}

		this.field_70165_t += this.field_70159_w * 3.0D;
		this.field_70163_u += this.field_70181_x * 3.0D;
		this.field_70161_v += this.field_70179_y * 3.0D;

		this.field_70177_z = (float) ((Math.atan2(this.field_70159_w, this.field_70179_y) * 180D) / 3.1415927410125732D);

		this.field_70125_A = this.field_70127_C + ((this.field_70125_A - this.field_70127_C) * 0.2F);
		this.field_70177_z = this.field_70126_B + ((this.field_70177_z - this.field_70126_B) * 0.2F);
		float speed = 0.99F;

		if (this.func_70072_I())
		{
			this.func_70106_y();
		}

		this.field_70159_w *= speed;
		this.field_70181_x *= speed;
		this.field_70179_y *= speed;
		this.func_70107_b(this.field_70165_t, this.field_70163_u, this.field_70161_v);

		if (this.field_70170_p.field_72995_K)
		{
			this.field_70170_p.func_175688_a(EnumParticleTypes.EXPLOSION_NORMAL, this.field_70165_t, this.field_70163_u, this.field_70161_v, 0, 0, 0);
		}
	}

	@Override
	public void func_70014_b(NBTTagCompound tagCompound)
	{
		tagCompound.func_74777_a("xTile", (short) this.xTile);
		tagCompound.func_74777_a("yTile", (short) this.yTile);
		tagCompound.func_74777_a("zTile", (short) this.zTile);
	}

	@Override
	public void func_70037_a(NBTTagCompound tagCompound)
	{
		this.xTile = tagCompound.func_74765_d("xTile");
		this.yTile = tagCompound.func_74765_d("yTile");
		this.zTile = tagCompound.func_74765_d("zTile");
	}

	@Override
	protected boolean func_70041_e_()
	{
		return false;
	}

	/*TODO Shadows?
	@Override
	@SideOnly(Side.CLIENT)
	public float getShadowSize()
	{
		return 0.0F;
	}*/

	@Override
	public boolean func_70075_an()
	{
		return false;
	}

	@Override
	public void func_70088_a()
	{
	}

	public abstract void onHitEntity(Entity entity);

	public abstract void onHitBlock(Block block, MovingObjectPosition mop);
}
